/*
*  Copyright (c) 2005-2010, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
*
*  WSO2 Inc. licenses this file to you under the Apache License,
*  Version 2.0 (the "License"); you may not use this file except
*  in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/
package org.wso2.carbon.mediation.initializer.persistence;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMException;
import org.apache.axiom.om.impl.llom.util.AXIOMUtil;
import org.apache.synapse.SynapseConstants;
import org.apache.synapse.config.SynapseConfiguration;
import org.apache.synapse.config.xml.MultiXMLConfigurationBuilder;
import org.apache.synapse.config.xml.TemplateMediatorSerializer;
import org.apache.synapse.config.xml.XMLConfigConstants;
import org.apache.synapse.config.xml.endpoints.TemplateSerializer;
import org.apache.synapse.endpoints.Template;
import org.apache.synapse.mediators.template.TemplateMediator;
import org.wso2.carbon.mediation.initializer.persistence.registry.TemplateRegistryStore;
import org.wso2.carbon.registry.core.session.UserRegistry;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;
import java.io.File;

public class EndpointTemplateStore extends AbstractStore<Template> {

    public EndpointTemplateStore(String configPath, UserRegistry registry, String configName) {
        super(configPath);
        if (registry != null) {
            this.registryStore = new TemplateRegistryStore(registry, configName);
        }
    }

    protected String getFileName(Template template) {
        return template.getFileName();
    }

    protected Template getObjectToPersist(String name, SynapseConfiguration config) {
        return config.getEndpointTemplates().get(name);
    }

    protected OMElement saveToFile(Template template, SynapseConfiguration synapseConfig) {
        try {
            return serializer.serializeTemplate(template, synapseConfig, null) ;
        } catch (Exception e) {
            handleException("Error while saving the template: " + template.getName() + " to the " +
                    "file system", e);
        }
        return null;
    }

    protected OMElement serialize(Template endpointTemplate) {
        TemplateSerializer serializer = new TemplateSerializer();
//        OMElement parent = OMAbstractFactory.getOMFactory().createOMElement(new QName(XMLConfigConstants.SYNAPSE_NAMESPACE,"parent"));
        return serializer.serializeEndpointTemplate(endpointTemplate, null);
    }

    @SuppressWarnings({"ResultOfMethodCallIgnored"})
    protected void deleteFile(String fileName, SynapseConfiguration synapseConfig) {
        File templateDir = new File(configPath, MultiXMLConfigurationBuilder.TEMPLATES_DIR);
        if (!templateDir.exists()) {
            return;
        }

        File templateFile = new File(templateDir, fileName);
        synapseConfig.getArtifactDeploymentStore().addBackedUpArtifact(
                templateFile.getAbsolutePath());
        templateFile.delete();
    }

}
